// DlgSamp.cs -- Show the use of some of the OpenFile dialog box.
//
//               Compile this program with the following command line:
//                   C:>csc DlgSamp.cs
using System;
using System.Windows.Forms;
using System.ComponentModel;

class clsMain
{
    static public void Main ()
    {
// Create the dialog box object.
        OpenFileDialog ofd = new OpenFileDialog ();
// Allow multiple file selection.
        ofd.Multiselect = true;
// Set the text for the title bar.
        ofd.Title = "Concatenate files";
// Do not verify that the file exists.
        ofd.CheckFileExists = false;
// Do verify that the path exists.
        ofd.CheckPathExists = true;
// Add a default extension if the user does not type one.
        ofd.AddExtension = true;
// Set the default extension.
        ofd.DefaultExt = "txt";
// Show the read-only box.
        ofd.ShowReadOnly = true;
// Show the Help button.
        ofd.ShowHelp = true;
// Call this method when the user clicks the OK (Open) button.
        ofd.FileOk += new CancelEventHandler (CancelOpenFile);
// Call this method when the user clicks the Help button.
        ofd.HelpRequest += new EventHandler (ShowOpenHelp);
// Show the dialog box.
        if (ofd.ShowDialog () == DialogResult.Cancel)
            return;
// Display a list of the selected files.
        foreach (string str in ofd.FileNames)
            Console.WriteLine (str);
    }
// Delegate called when the user clicks the OK (Open) button
    static private void CancelOpenFile (object sender, CancelEventArgs e)
    {
// Cast the object to an OpenFileDialog object.
        OpenFileDialog dlg = (OpenFileDialog) sender;
// Show the selected files.
        Console.WriteLine ("The selected file are:");
        foreach (string str in dlg.FileNames)
            Console.WriteLine ("\t" + str);
// Ask whether to cancel the close event.
        Console.Write ("\r\nCancel event? [y/n]: ");
        string reply = Console.ReadLine ();
        if (reply[0] == 'y')
            e.Cancel = true;
    }
// Delegate called when the user clicks the Help button.
    static private void ShowOpenHelp (object sender, EventArgs e)
    {
        Console.WriteLine ("Open your help file to the File Open topic here.");
    }
}
