//  Circle.cs -- File to be used as a library assembly
//
//               Compile this file with the following command line:
//                   C:>csc /t:library Circle.cs
using System;

namespace nsCircle
{
// A structure to define a Cartesian point.
    public struct POINT
    {
        public POINT (int x, int y)
        {
            cx = x;
            cy = y;
        }
        public int  cx;
        public int  cy;
        public override string ToString()
        {
           return (String.Format ("(" + cx + ", " + cy + ")"));
        }
    }
    public class clsCircle
    {
// Two constructors to define the circle.
        public clsCircle (double radius, POINT center)
        {
            m_Center = center;
            m_Radius = radius;
        }
        public clsCircle (double radius, int cx, int cy)
        {
            m_Center.cx = cx;
            m_Center.cy = cy;
            m_Radius = radius;
        }
        public clsCircle ()
        {
            m_Center.cx = 0;
            m_Center.cy = 0;
            m_Radius = 0;
        }
        public double Radius
        {
            get {return (m_Radius);}
            set {m_Radius = value;}
        }
        public POINT Center
        {
            get {return (m_Center);}
            set {m_Center = value;}
        }
// Fields to contain circle data.
        POINT m_Center;
        private double m_Radius;

// Constants to make life easier
        private const double pi = 3.14159;
        private const double radian = 57.29578;
// Return the area of the circle
        public double Area
        {
            get {return (m_Radius * m_Radius * pi);}
        }
// Return the diameter of the circle
        public double Diameter
        {
            get {return (2 * m_Radius);}
        }
// Return the coordinates of a point on the circle at a given angle.
        public POINT PointOnCircle (double degrees)
        {
            POINT pt;
            double fAngle = degrees / radian;
// Compute the x position of the point
            pt.cx = (int)((double) m_Radius * Math.Cos (fAngle) + 0.5);
// Compute the y position of the point
            pt.cy = (int)((double) m_Radius * Math.Sin (fAngle) + 0.5);
            return (pt);
        }
// Return the area of a slice determined by a given angle.
        public double AreaOfSlice (double degrees)
        {
            double fAngle = degrees / 57.29578;
            return (Area * fAngle / (2 * pi));
        }
    }
}
